@@ -203,7 +203,7 @@ function _cidr_range {
 import sys
 from netaddr import IPAddress, IPNetwork
 n = IPNetwork(sys.argv[1])
-print("%s\\t%s" % (IPAddress(n.first + 1), IPAddress(n.last - 1)))
+print("%s\\t%s\\t%s" % (IPAddress(n.first + 1), IPAddress(n.first + 2), IPAddress(n.last - 1)))
 EOF
 }
 
@@ -454,14 +454,14 @@ function configure_neutron_defaults {
         # NOTE(ltomasbo): As more security groups and rules are created, there
         # is a need to increase the quota for it
          openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-             quota set --secgroups 100 --secgroup-rules 100 "$project_id"
+             quota set --secgroups 100 --secgroup-rules 300 "$project_id"
     fi
 
     # NOTE(dulek): DevStack's admin default for SG's and instances is 10, this
     #              is too little for our tests with Octavia configured to use
     #              amphora.
     openstack --os-cloud devstack-admin --os-region "$REGION_NAME" \
-        quota set --secgroups 100 --secgroup-rules 100 --instances 100 admin
+        quota set --secgroups 100 --secgroup-rules 300 --instances 100 admin
 
     if [ -n "$OVS_BRIDGE" ]; then
         iniset "$KURYR_CONFIG" neutron_defaults ovs_bridge "$OVS_BRIDGE"
@@ -518,20 +518,20 @@ function prepare_kubernetes_files {
     sudo groupadd -f -r kube-cert
 
     # hostname -I gets the ip of the node
-    sudo CERT_DIR=${KURYR_HYPERKUBE_DATA_DIR} /tmp/make-ca-cert.sh $(hostname -I | awk '{print $1}') "IP:${HOST_IP},IP:${k8s_api_clusterip},DNS:kubernetes,DNS:kubernetes.default,DNS:kubernetes.default.svc,DNS:kubernetes.default.svc.cluster.local"
+    sudo CERT_DIR=${KURYR_KUBERNETES_DATA_DIR} /tmp/make-ca-cert.sh $(hostname -I | awk '{print $1}') "IP:${HOST_IP},IP:${k8s_api_clusterip},DNS:kubernetes,DNS:kubernetes.default,DNS:kubernetes.default.svc,DNS:kubernetes.default.svc.cluster.local"
 
     # Create basic token authorization
-    sudo bash -c "echo 'admin,admin,admin' > $KURYR_HYPERKUBE_DATA_DIR/basic_auth.csv"
+    sudo bash -c "echo 'admin,admin,admin' > $KURYR_KUBERNETES_DATA_DIR/basic_auth.csv"
 
     # Create known tokens for service accounts
-    sudo bash -c "echo '$(create_token),admin,admin' >> ${KURYR_HYPERKUBE_DATA_DIR}/known_tokens.csv"
-    sudo bash -c "echo '$(create_token),kubelet,kubelet' >> ${KURYR_HYPERKUBE_DATA_DIR}/known_tokens.csv"
-    sudo bash -c "echo '$(create_token),kube_proxy,kube_proxy' >> ${KURYR_HYPERKUBE_DATA_DIR}/known_tokens.csv"
+    sudo bash -c "echo '$(create_token),admin,admin' >> ${KURYR_KUBERNETES_DATA_DIR}/known_tokens.csv"
+    sudo bash -c "echo '$(create_token),kubelet,kubelet' >> ${KURYR_KUBERNETES_DATA_DIR}/known_tokens.csv"
+    sudo bash -c "echo '$(create_token),kube_proxy,kube_proxy' >> ${KURYR_KUBERNETES_DATA_DIR}/known_tokens.csv"
 
     # Copy certs for Kuryr services to use
-    sudo install -m 644 "${KURYR_HYPERKUBE_DATA_DIR}/kubecfg.crt" "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.crt"
-    sudo install -m 644 "${KURYR_HYPERKUBE_DATA_DIR}/kubecfg.key" "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.key"
-    sudo install -m 644 "${KURYR_HYPERKUBE_DATA_DIR}/ca.crt" "${KURYR_HYPERKUBE_DATA_DIR}/kuryr-ca.crt"
+    sudo install -m 644 "${KURYR_KUBERNETES_DATA_DIR}/kubecfg.crt" "${KURYR_KUBERNETES_DATA_DIR}/kuryr.crt"
+    sudo install -m 644 "${KURYR_KUBERNETES_DATA_DIR}/kubecfg.key" "${KURYR_KUBERNETES_DATA_DIR}/kuryr.key"
+    sudo install -m 644 "${KURYR_KUBERNETES_DATA_DIR}/ca.crt" "${KURYR_KUBERNETES_DATA_DIR}/kuryr-ca.crt"
 
     # FIXME(ivc): replace 'sleep' with a strict check (e.g. wait_for_files)
     # 'kubernetes-api' fails if started before files are generated.
@@ -603,10 +603,23 @@ function get_k8s_log_level {
   fi
 }
 
+function setup_k8s_binaries() {
+    tmp_path=$1
+    binary_name=$2
+    binary_path=$3
+
+    curl -o "${tmp_path}" "${KURYR_KUBERNETES_BINARIES}/${binary_name}"
+    sudo install -o "$STACK_USER" -m 0555 -D "${tmp_path}" "${binary_path}"
+}
+
 function run_k8s_api {
     local service_cidr
     local cluster_ip_range
     local command
+    local tmp_kube_apiserver_path="/tmp/kube-apiserver"
+    local binary_name="kube-apiserver"
+
+    setup_k8s_binaries $tmp_kube_apiserver_path $binary_name $KURYR_KUBE_APISERVER_BINARY
 
     # Runs Hyperkube's Kubernetes API Server
     wait_for "etcd" "http://${SERVICE_HOST}:${ETCD_PORT}/v2/machines"
@@ -621,119 +634,86 @@ function run_k8s_api {
         cluster_ip_range="$service_cidr"
     fi
 
-    command=(--net=host
-             --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
-    if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
-        command+=(--restart=on-failure)
-    fi
-    command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
-              /hyperkube kube-apiserver
-                --service-cluster-ip-range=${cluster_ip_range}
-                --insecure-bind-address=0.0.0.0
-                --insecure-port=${KURYR_K8S_API_PORT}
-                --etcd-servers=http://${SERVICE_HOST}:${ETCD_PORT}
-                --client-ca-file=/srv/kubernetes/ca.crt
-                --basic-auth-file=/srv/kubernetes/basic_auth.csv
-                --min-request-timeout=300
-                --tls-cert-file=/srv/kubernetes/server.cert
-                --tls-private-key-file=/srv/kubernetes/server.key
-                --token-auth-file=/srv/kubernetes/known_tokens.csv
-                --allow-privileged=true
-                --v=$(get_k8s_log_level)
-                --logtostderr=true)
+    command="${KURYR_KUBE_APISERVER_BINARY} \
+                --service-cluster-ip-range=${cluster_ip_range} \
+                --insecure-bind-address=0.0.0.0 \
+                --insecure-port=${KURYR_K8S_API_PORT} \
+                --etcd-servers=http://${SERVICE_HOST}:${ETCD_PORT} \
+                --client-ca-file=${KURYR_KUBERNETES_DATA_DIR}/ca.crt \
+                --basic-auth-file=${KURYR_KUBERNETES_DATA_DIR}/basic_auth.csv \
+                --min-request-timeout=300 \
+                --tls-cert-file=${KURYR_KUBERNETES_DATA_DIR}/server.cert \
+                --tls-private-key-file=${KURYR_KUBERNETES_DATA_DIR}/server.key \
+                --token-auth-file=${KURYR_KUBERNETES_DATA_DIR}/known_tokens.csv \
+                --allow-privileged=true \
+                --v=$(get_k8s_log_level) \
+                --logtostderr=true"
 
-    run_container kubernetes-api "${command[@]}"
+    run_process kubernetes-api "$command" root root
 }
 
 function run_k8s_controller_manager {
     local command
+    local tmp_kube_controller_manager="/tmp/kube-controller-manager"
+    local binary_name="kube-controller-manager"
+
+    setup_k8s_binaries $tmp_kube_controller_manager $binary_name $KURYR_KUBE_CONTROLLER_MANAGER_BINARY
 
     # Runs Hyperkube's Kubernetes controller manager
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
 
-    command=(--net=host
-             --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
-    if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
-        command+=(--restart=on-failure)
-    fi
-    command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
-              /hyperkube kube-controller-manager
-                --master=$KURYR_K8S_API_URL
-                --service-account-private-key-file=/srv/kubernetes/server.key
-                --root-ca-file=/srv/kubernetes/ca.crt
-                --min-resync-period=3m
-                --v=$(get_k8s_log_level)
-                --logtostderr=true
-                --leader-elect=false)
+    command="${KURYR_KUBE_CONTROLLER_MANAGER_BINARY} \
+                --master=$KURYR_K8S_API_URL \
+                --service-account-private-key-file=${KURYR_KUBERNETES_DATA_DIR}/server.key \
+                --root-ca-file=${KURYR_KUBERNETES_DATA_DIR}/ca.crt \
+                --min-resync-period=3m \
+                --v=$(get_k8s_log_level) \
+                --logtostderr=true \
+                --leader-elect=false"
 
-    run_container kubernetes-controller-manager "${command[@]}"
+    run_process kubernetes-controller-manager "$command" root root
 }
 
 function run_k8s_scheduler {
     local command
+    local tmp_kube_scheduler="/tmp/kube-scheduler"
+    local binary_name="kube-scheduler"
 
-    # Runs Hyperkube's Kubernetes scheduler
+    setup_k8s_binaries $tmp_kube_scheduler $binary_name $KURYR_KUBE_SCHEDULER_BINARY
+
+    # Runs Kubernetes scheduler
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
 
-    command=(--net=host
-             --volume=${KURYR_HYPERKUBE_DATA_DIR}:/srv/kubernetes:rw)
-    if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
-        command+=(--restart=on-failure)
-    fi
-    command+=(${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}
-              /hyperkube kube-scheduler
-                --master=$KURYR_K8S_API_URL
-                --v=$(get_k8s_log_level)
-                --logtostderr=true
-                --leader-elect=false)
+    command="${KURYR_KUBE_SCHEDULER_BINARY} \
+                --master=${KURYR_K8S_API_URL} \
+                --v=$(get_k8s_log_level) \
+                --logtostderr=true \
+                --leader-elect=false"
 
-    run_container kubernetes-scheduler "${command[@]}"
+    run_process kubernetes-scheduler "$command" root root
 }
 
 function prepare_kubeconfig {
-    $KURYR_HYPERKUBE_BINARY kubectl config set-cluster devstack-cluster \
+    $KURYR_KUBECTL_BINARY config set-cluster devstack-cluster \
         --server="${KURYR_K8S_API_URL}"
-    $KURYR_HYPERKUBE_BINARY kubectl config set-context devstack \
-        --cluster=devstack-cluster
-    $KURYR_HYPERKUBE_BINARY kubectl config use-context devstack
+    $KURYR_KUBECTL_BINARY config set-credentials stack
+    $KURYR_KUBECTL_BINARY config set-context devstack \
+        --cluster=devstack-cluster --user=stack
+    $KURYR_KUBECTL_BINARY config use-context devstack
 }
 
-function extract_hyperkube {
-    local hyperkube_container
-    local tmp_hyperkube_path
+function extract_k8s_binaries {
+    local tmp_kubectl_path="/tmp/kubectl"
+    local tmp_kubelet_path="/tmp/kubelet"
+    local tmp_loopback_cni_path="/tmp/loopback"
+    local kubectl_binary_name="kubectl"
+    local kubelet_binary_name="kubelet"
 
-    tmp_hyperkube_path="/tmp/hyperkube"
-    tmp_loopback_cni_path="/tmp/loopback"
-    tmp_nsenter_path="/tmp/nsenter"
+    setup_k8s_binaries $tmp_kubectl_path $kubectl_binary_name $KURYR_KUBECTL_BINARY
+    setup_k8s_binaries $tmp_kubelet_path $kubelet_binary_name $KURYR_KUBELET_BINARY
 
-    hyperkube_container=$(container_runtime run -d \
-        --net host \
-       "${KURYR_HYPERKUBE_IMAGE}:${KURYR_HYPERKUBE_VERSION}" \
-       /bin/false)
-    if [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
-        mnt=`container_runtime mount "${hyperkube_container}"`
-        sudo cp "${mnt}/hyperkube" "$tmp_hyperkube_path"
-        sudo cp "${mnt}/opt/cni/bin/loopback" "$tmp_loopback_cni_path"
-        sudo cp "${mnt}/usr/bin/nsenter" "$tmp_nsenter_path"
-        container_runtime umount ${hyperkube_container}
-    else
-        container_runtime cp "${hyperkube_container}:/hyperkube" "$tmp_hyperkube_path"
-        container_runtime cp "${hyperkube_container}:/opt/cni/bin/loopback" \
-            "$tmp_loopback_cni_path"
-        container_runtime cp "${hyperkube_container}:/usr/bin/nsenter" "$tmp_nsenter_path"
-    fi
-
-    container_runtime rm --force "$hyperkube_container"
-    sudo install -o "$STACK_USER" -m 0555 -D "$tmp_hyperkube_path" \
-        "$KURYR_HYPERKUBE_BINARY"
-    sudo install -o "$STACK_USER" -m 0555 -D "$tmp_loopback_cni_path" \
-        "${CNI_BIN_DIR}/loopback"
-    sudo install -o "root" -m 0555 -D "$tmp_nsenter_path" \
-        "/usr/local/bin/nsenter"
-
-    # Convenience kubectl executable for development
-    sudo install -o "$STACK_USER" -m 555 -D "${KURYR_HOME}/devstack/kubectl" \
-        "$(dirname $KURYR_HYPERKUBE_BINARY)/kubectl"
+    sudo mkdir -p "$CNI_BIN_DIR"
+    curl -L "${KURYR_CNI_PLUGINS}" | sudo tar -C "${CNI_BIN_DIR}" -xzvf - ./loopback
 }
 
 function prepare_kubelet {
@@ -741,8 +721,8 @@ function prepare_kubelet {
     kubelet_plugin_dir="/etc/cni/net.d/"
 
     sudo install -o "$STACK_USER" -m 0664 -D \
-        "${KURYR_HOME}${kubelet_plugin_dir}/10-kuryr.conf" \
-        "${CNI_CONF_DIR}/10-kuryr.conf"
+        "${KURYR_HOME}${kubelet_plugin_dir}/10-kuryr.conflist" \
+        "${CNI_CONF_DIR}/10-kuryr.conflist"
 }
 
 function run_k8s_kubelet {
@@ -756,8 +736,8 @@ function run_k8s_kubelet {
     local command
     local minor_version
 
-    sudo mkdir -p "${KURYR_HYPERKUBE_DATA_DIR}/"{kubelet,kubelet.cert}
-    command="$KURYR_HYPERKUBE_BINARY kubelet\
+    sudo mkdir -p "${KURYR_KUBERNETES_DATA_DIR}/"{kubelet,kubelet.cert}
+    command="$KURYR_KUBELET_BINARY \
         --kubeconfig=${HOME}/.kube/config \
         --v=2 \
         --address=0.0.0.0 \
@@ -765,11 +745,11 @@ function run_k8s_kubelet {
         --network-plugin=cni \
         --cni-bin-dir=$CNI_BIN_DIR \
         --cni-conf-dir=$CNI_CONF_DIR \
-        --cert-dir=${KURYR_HYPERKUBE_DATA_DIR}/kubelet.cert \
-        --root-dir=${KURYR_HYPERKUBE_DATA_DIR}/kubelet"
+        --cert-dir=${KURYR_KUBERNETES_DATA_DIR}/kubelet.cert \
+        --root-dir=${KURYR_KUBERNETES_DATA_DIR}/kubelet"
 
     if [[ ${CONTAINER_ENGINE} == 'docker' ]]; then
-        command+=" --cgroup-driver $(docker info -f '{{.CgroupDriver}}')"
+        command+=" --cgroup-driver $(docker info|awk '/Cgroup/ {print $NF}')"
     elif [[ ${CONTAINER_ENGINE} == 'crio' ]]; then
         local crio_conf
         crio_conf=/etc/crio/crio.conf
@@ -785,20 +765,24 @@ function run_k8s_kubelet {
     fi
 
     declare -r min_not_require_kubeconfig_ver="1.10.0"
-    if [[ "$KURYR_HYPERKUBE_VERSION" == "$(echo -e "${KURYR_HYPERKUBE_VERSION}\n${min_not_require_kubeconfig_ver}" | sort -V | head -n 1)" ]]; then
+    if [[ "$KURYR_KUBERNETES_VERSION" == "$(echo -e "${KURYR_KUBERNETES_VERSION}\n${min_not_require_kubeconfig_ver}" | sort -V | head -n 1)" ]]; then
         # Version 1.10 did away with that config option
         command+=" --require-kubeconfig"
     fi
 
     # Kubernetes 1.8+ requires additional option to work in the gate.
     declare -r min_no_swap_ver="1.8.0"
-    if [[ "$min_no_swap_ver" == "$(echo -e "${KURYR_HYPERKUBE_VERSION}\n${min_no_swap_ver}" | sort -V | head -n 1)" ]]; then
+    if [[ "$min_no_swap_ver" == "$(echo -e "${KURYR_KUBERNETES_VERSION}\n${min_no_swap_ver}" | sort -V | head -n 1)" ]]; then
         command="$command --fail-swap-on=false"
     fi
 
     if is_service_enabled coredns; then
-        local k8s_resolv_conf
-        command+=" --cluster-dns=${HOST_IP} --cluster-domain=cluster.local"
+        service_cidr=$(openstack --os-cloud devstack-admin \
+                                 --os-region "$REGION_NAME" \
+                                 subnet show "$KURYR_NEUTRON_DEFAULT_SERVICE_SUBNET" \
+                                 -c cidr -f value)
+        export KURYR_COREDNS_CLUSTER_IP=$(_cidr_range "$service_cidr" | cut -f2)
+        command+=" --cluster-dns=${KURYR_COREDNS_CLUSTER_IP} --cluster-domain=cluster.local"
     fi
 
     wait_for "Kubernetes API Server" "$KURYR_K8S_API_URL"
@@ -818,14 +802,13 @@ metadata:
 data:
   Corefile: |
     .:53 {
-        bind ${HOST_IP}
         errors
         kubernetes cluster.local in-addr.arpa ip6.arpa {
            pods insecure
            upstream
            fallthrough in-addr.arpa ip6.arpa
         }
-        proxy . /etc/resolv.conf
+        forward . 8.8.8.8:53
         cache 30
         loop
         reload
@@ -862,10 +845,9 @@ spec:
         scheduler.alpha.kubernetes.io/critical-pod: ''
         scheduler.alpha.kubernetes.io/tolerations: '[{"key":"CriticalAddonsOnly", "operator":"Exists"}]'
     spec:
-      hostNetwork: true
       containers:
       - name: coredns
-        image: quay.io/kuryr/coredns:1.4.0
+        image: coredns/coredns:1.5.0
         imagePullPolicy: Always
         args: [ "-conf", "/etc/coredns/Corefile" ]
         volumeMounts:
@@ -882,6 +864,7 @@ spec:
 EOF
 
     /usr/local/bin/kubectl apply -f ${output_dir}/coredns.yml
+    /usr/local/bin/kubectl expose deploy/coredns --port=53 --target-port=53 --protocol=UDP -n kube-system --cluster-ip=${KURYR_COREDNS_CLUSTER_IP}
 }
 
 
@@ -893,9 +876,9 @@ function run_kuryr_kubernetes {
             "${OPENSHIFT_DATA_DIR}/master/ca.crt" 1200
     else
         wait_for_ok_health "Kubernetes API Server" "${KURYR_K8S_API_ROOT}/healthz" \
-            "${KURYR_HYPERKUBE_DATA_DIR}/kuryr-ca.crt" \
-            "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.key" \
-            "${KURYR_HYPERKUBE_DATA_DIR}/kuryr.crt" \
+            "${KURYR_KUBERNETES_DATA_DIR}/kuryr-ca.crt" \
+            "${KURYR_KUBERNETES_DATA_DIR}/kuryr.key" \
+            "${KURYR_KUBERNETES_DATA_DIR}/kuryr.crt" \
             1200
     fi
 
@@ -972,6 +955,9 @@ function update_tempest_conf_file {
     fi
     iniset $TEMPEST_CONFIG kuryr_kubernetes validate_crd True
     iniset $TEMPEST_CONFIG kuryr_kubernetes kuryrnetworks True
+    iniset $TEMPEST_CONFIG kuryr_kubernetes kuryrports True
+    iniset $TEMPEST_CONFIG kuryr_kubernetes kuryrloadbalancers True
+    iniset $TEMPEST_CONFIG kuryr_kubernetes new_kuryrnetworkpolicy_crd True
 }
 
 source $DEST/kuryr-kubernetes/devstack/lib/kuryr_kubernetes
@@ -1067,7 +1053,7 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
        || is_service_enabled kubernetes-controller-manager \
        || is_service_enabled kubernetes-scheduler \
        || is_service_enabled kubelet; then
-        get_container "$KURYR_HYPERKUBE_IMAGE" "$KURYR_HYPERKUBE_VERSION"
+        get_container "$KURYR_KUBERNETES_IMAGE" "$KURYR_KUBERNETES_VERSION"
         prepare_kubernetes_files
     fi
 
@@ -1085,7 +1071,7 @@ if [[ "$1" == "stack" && "$2" == "extra" ]]; then
 
     if is_service_enabled kubelet; then
         prepare_kubelet
-        extract_hyperkube
+        extract_k8s_binaries
         prepare_kubeconfig
         run_k8s_kubelet
         KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE=$(trueorfalse True KURYR_CONFIGURE_BAREMETAL_KUBELET_IFACE)
@@ -1139,12 +1125,8 @@ elif [[ "$1" == "stack" && "$2" == "test-config" ]]; then
         fi
 
         if is_service_enabled coredns; then
-            #Open port 53 so pods can reach the DNS server
-            sudo iptables -I INPUT 1 -p udp -m udp --dport 53 -j ACCEPT
-
             run_coredns "${DATA_DIR}/kuryr-kubernetes"
         fi
-
         # Needs kuryr to be running
         if is_service_enabled openshift-dns; then
             configure_and_run_registry
@@ -1159,14 +1141,14 @@ if [[ "$1" == "unstack" ]]; then
     KURYR_K8S_CONTAINERIZED_DEPLOYMENT=$(trueorfalse False KURYR_K8S_CONTAINERIZED_DEPLOYMENT)
     if is_service_enabled kuryr-kubernetes; then
         if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
-            $KURYR_HYPERKUBE_BINARY kubectl delete deploy/kuryr-controller
+            kubectl delete deploy/kuryr-controller
         fi
         stop_process kuryr-kubernetes
     elif is_service_enabled kubelet; then
-         $KURYR_HYPERKUBE_BINARY kubectl delete nodes ${HOSTNAME}
+        kubectl delete nodes ${HOSTNAME}
     fi
     if [ "$KURYR_K8S_CONTAINERIZED_DEPLOYMENT" == "True" ]; then
-        $KURYR_HYPERKUBE_BINARY kubectl delete ds/kuryr-cni-ds
+        kubectl delete ds/kuryr-cni-ds
     fi
     stop_process kuryr-daemon
 
