@@ -13,20 +13,21 @@
 # limitations under the License.
 
 from ctypes import c_bool
+from http import client as httplib
 import multiprocessing
 import os
-from six.moves import http_client as httplib
 import socket
 import sys
 import threading
 import time
+import urllib.parse
+import urllib3
 
 import cotyledon
 import flask
 from pyroute2.ipdb import transactional
 
 import os_vif
-from os_vif.objects import base
 from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -103,10 +104,10 @@ class DaemonServer(object):
             self.plugin.delete(params)
         except exceptions.ResourceNotReady:
             # NOTE(dulek): It's better to ignore this error - most of the time
-            #              it will happen when pod is long gone and kubelet
+            #              it will happen when pod is long gone and CRI
             #              overzealously tries to delete it from the network.
             #              We cannot really do anything without VIF annotation,
-            #              so let's just tell kubelet to move along.
+            #              so let's just tell CRI to move along.
             LOG.warning('Error when processing delNetwork request. '
                         'Ignoring this error, pod is most likely gone')
             return '', httplib.NO_CONTENT, self.headers
@@ -150,7 +151,6 @@ class CNIDaemonServerService(cotyledon.Service):
 
     def __init__(self, worker_id, registry, healthy):
         super(CNIDaemonServerService, self).__init__(worker_id)
-        self.run_queue_reading = False
         self.registry = registry
         self.healthy = healthy
         self.plugin = k8s_cni_registry.K8sCNIRegistryPlugin(registry,
@@ -193,10 +193,12 @@ class CNIDaemonWatcherService(cotyledon.Service):
         self.pipeline.register(h_cni.CallbackHandler(self.on_done,
                                                      self.on_deleted))
         self.watcher = k_watcher.Watcher(self.pipeline)
-        self.watcher.add(
-            "%(base)s/pods?fieldSelector=spec.nodeName=%(node_name)s" % {
-                'base': k_const.K8S_API_BASE,
-                'node_name': self._get_nodename()})
+        query_label = urllib.parse.quote_plus(f'{k_const.KURYRPORT_LABEL}='
+                                              f'{self._get_nodename()}')
+
+        self.watcher.add(f'{k_const.K8S_API_CRD_KURYRPORTS}'
+                         f'?labelSelector={query_label}')
+
         self.is_running = True
         self.health_thread = threading.Thread(
             target=self._start_watcher_health_checker)
@@ -211,43 +213,43 @@ class CNIDaemonWatcherService(cotyledon.Service):
                     self.healthy.value = False
             time.sleep(HEALTH_CHECKER_DELAY)
 
-    def on_done(self, pod, vifs):
-        pod_name = utils.get_pod_unique_name(pod)
-        vif_dict = {
-            ifname: vif.obj_to_primitive() for
-            ifname, vif in vifs.items()
-        }
-        # NOTE(dulek): We need a lock when modifying shared self.registry dict
-        #              to prevent race conditions with other processes/threads.
-        with lockutils.lock(pod_name, external=True):
-            if pod_name not in self.registry:
-                self.registry[pod_name] = {'pod': pod, 'vifs': vif_dict,
-                                           'containerid': None}
+    def on_done(self, kuryrport, vifs):
+        kp_name = utils.get_res_unique_name(kuryrport)
+        with lockutils.lock(kp_name, external=True):
+            if (kp_name not in self.registry or
+                    self.registry[kp_name]['kp']['metadata']['uid']
+                    != kuryrport['metadata']['uid']):
+                self.registry[kp_name] = {'kp': kuryrport,
+                                          'vifs': vifs,
+                                          'containerid': None,
+                                          'vif_unplugged': False,
+                                          'del_received': False}
             else:
-                # NOTE(dulek): Only update vif if its status changed, we don't
-                #              need to care about other changes now.
-                old_vifs = {
-                    ifname:
-                        base.VersionedObject.obj_from_primitive(vif_obj) for
-                        ifname, vif_obj in (
-                            self.registry[pod_name]['vifs'].items())
-                }
+                old_vifs = self.registry[kp_name]['vifs']
                 for iface in vifs:
                     if old_vifs[iface].active != vifs[iface].active:
-                        pod_dict = self.registry[pod_name]
-                        pod_dict['vifs'] = vif_dict
-                        self.registry[pod_name] = pod_dict
+                        kp_dict = self.registry[kp_name]
+                        kp_dict['vifs'] = vifs
+                        self.registry[kp_name] = kp_dict
 
-    def on_deleted(self, pod):
-        pod_name = utils.get_pod_unique_name(pod)
+    def on_deleted(self, kp):
+        kp_name = utils.get_res_unique_name(kp)
         try:
-            if pod_name in self.registry:
-                # NOTE(dulek): del on dict is atomic as long as we use standard
-                #              types as keys. This is the case, so we don't
-                #              need to lock here.
-                del self.registry[pod_name]
+            if kp_name in self.registry:
+                # NOTE(ndesh): We need to lock here to avoid race condition
+                #              with the deletion code for CNI DEL so that
+                #              we delete the registry entry exactly once
+                with lockutils.lock(kp_name, external=True):
+                    if self.registry[kp_name]['vif_unplugged']:
+                        del self.registry[kp_name]
+                    else:
+                        kp_dict = self.registry[kp_name]
+                        kp_dict['del_received'] = True
+                        self.registry[kp_name] = kp_dict
         except KeyError:
             # This means someone else removed it. It's odd but safe to ignore.
+            LOG.debug('KuryrPort %s entry already removed from registry while '
+                      'handling DELETED event. Ignoring.', kp_name)
             pass
 
     def terminate(self):
@@ -292,13 +294,36 @@ class CNIDaemonServiceManager(cotyledon.ServiceManager):
         self.register_hooks(on_terminate=self.terminate)
 
     def run(self):
+        reaper_thread = threading.Thread(target=self._zombie_reaper,
+                                         daemon=True)
+        self._terminate_called = threading.Event()
+        reaper_thread.start()
         super(CNIDaemonServiceManager, self).run()
 
+    def _zombie_reaper(self):
+        while True:
+            try:
+                res = os.waitpid(-1, os.WNOHANG)
+                # don't sleep or stop if a zombie process was found
+                # as there could be more
+                if res != (0, 0):
+                    continue
+            except ChildProcessError:
+                # There are no child processes yet (or they have been killed)
+                pass
+            except os.error:
+                LOG.exception("Got OS error while reaping zombie processes")
+            if self._terminate_called.isSet():
+                break
+            time.sleep(1)
+
     def terminate(self):
+        self._terminate_called.set()
         self.manager.shutdown()
 
 
 def start():
+    urllib3.disable_warnings()
     config.init(sys.argv[1:])
     config.setup_logging()
 
