@@ -87,13 +87,14 @@ k8s_opts = [
                       "connect to HTTPS K8S_API")),
     cfg.StrOpt('ssl_ca_crt_file',
                help=_("Absolute path to ca cert file to "
-                      "connect to HTTPS K8S_API")),
+                      "connect to HTTPS K8S_API"),
+               default='/var/run/secrets/kubernetes.io/serviceaccount/ca.crt'),
     cfg.BoolOpt('ssl_verify_server_crt',
                 help=_("HTTPS K8S_API server identity verification"),
                 default=False),
     cfg.StrOpt('token_file',
                help=_("The token to talk to the k8s API"),
-               default=''),
+               default='/var/run/secrets/kubernetes.io/serviceaccount/token'),
     cfg.StrOpt('pod_project_driver',
                help=_("The driver to determine OpenStack "
                       "project for pod ports"),
@@ -167,6 +168,14 @@ k8s_opts = [
                       'too high, Kuryr will take longer to reconnect when K8s '
                       'API stream was being silently broken.'),
                default=60),
+    cfg.IntOpt('watch_reconcile_period',
+               help=_('Period (in seconds) between iterations of fetching '
+                      'full list of watched K8s API resources and putting '
+                      'them into the enabled handlers. Setting 0 disables the '
+                      'periodic reconciling. The reconciliation is done to '
+                      'prevent Kuryr from missing events due to K8s API or '
+                      'etcd issues.'),
+               default=120),
     cfg.ListOpt('enabled_handlers',
                 help=_("The comma-separated handlers that should be "
                        "registered for watching in the pipeline."),
@@ -186,6 +195,10 @@ k8s_opts = [
                 help=_("The drivers that provide additional VIFs for "
                        "Kubernetes Pods."),
                 default='noop'),
+    cfg.StrOpt('additional_ifname_prefix',
+               help=_("The prefix to use for additional vifs created by "
+                      " multi_vif drivers"),
+               default='eth'),
 ]
 
 neutron_defaults = [
@@ -224,7 +237,7 @@ neutron_defaults = [
                        "This can be used to identify and garbage-collect "
                        "them when Kubernetes cluster Kuryr was serving is no "
                        "longer needed."),
-                default=[])
+                default=[]),
 ]
 
 octavia_defaults = [
@@ -236,12 +249,20 @@ octavia_defaults = [
                help=_("Define the LBaaS SG policy."),
                choices=[('create', 'replace the VIP SG with a new one'),
                         ('update', 'add rules to the existing VIP SG')],
-               default='update'),
+               default='update',
+               deprecated_for_removal=True,
+               deprecated_reason="enforce_sg_rules option can be used"
+                                 " instead"),
     cfg.BoolOpt('enforce_sg_rules',
                 help=_("Enable the enforcement of SG rules at the LB SG "
                        "in case the LB does not maintain the source IP "
                        "of the caller resource"),
                 default=True),
+    cfg.StrOpt('lb_algorithm',
+               help=_("The load-balancer algoritm that distributed traffic "
+                      "to the pool members. The options are: ROUND_ROBIN, "
+                      "LEAST_CONNECTIONS, SOURCE_IP and SOURCE_IP_PORT."),
+               default='ROUND_ROBIN'),
 ]
 
 cache_defaults = [
@@ -253,15 +274,14 @@ cache_defaults = [
                default="dogpile.cache.memory"),
 ]
 
-ingress = [
-    cfg.StrOpt('l7_router_uuid',
-               help=_("UUID of the L7 Router")),
-]
-
 nested_vif_driver_opts = [
     cfg.StrOpt('worker_nodes_subnet',
                help=_("Neutron subnet ID for k8s worker node vms."),
                default=''),
+    cfg.IntOpt('rev_update_attempts',
+               help=_("How many time to try to re-update the neutron resource "
+                      "when revision has been changed by other thread"),
+               default=3),
 ]
 
 DEFAULT_PHYSNET_SUBNET_MAPPINGS = {}
@@ -277,13 +297,6 @@ sriov_opts = [
                 help=_("A mapping of default subnets for certain physnets "
                        "in a form of physnet-name:<SUBNET-ID>"),
                 default=DEFAULT_PHYSNET_SUBNET_MAPPINGS),
-    cfg.ListOpt('physical_device_mappings',
-                default=DEFAULT_DEVICE_MAPPINGS,
-                help=_("Comma-separated list of "
-                       "<physical_network>:<network_device> tuples mapping "
-                       "physical network names to the agent's node-specific "
-                       "physical network device interfaces of SR-IOV physical "
-                       "function to be used for VLAN networks.")),
     cfg.DictOpt('physnet_resource_mappings',
                 help=_("A mapping of physnets for certain sriov dp "
                        "resource name in a form of "
@@ -301,9 +314,26 @@ sriov_opts = [
                        "names. Expected that device of VIF related to "
                        "exact physnet should be binded on specified driver."),
                 default=DEFAULT_PHYSNET_SUBNET_MAPPINGS),
+    cfg.BoolOpt('enable_node_annotations',
+                help=_("Enable node annotations. This option allows to "
+                       "set annotations required by neutron to set active "
+                       "state of ports. This option is useless when "
+                       "sriov-nic-agent is not running on node."),
+                default=False),
 ]
 
 
+vhostuser = [
+    cfg.StrOpt('mount_point',
+               help=_("Path where vhost-user port will be created "
+                      "also it should be mount point for pod"),
+               default='/var/cni/vhostuser'),
+    cfg.StrOpt('ovs_vhu_path',
+               help=_("Path where OVS keeps socket files for vhost-user "
+                      "ports"),
+               default='/var/run/openvswitch/')
+]
+
 CONF = cfg.CONF
 CONF.register_opts(kuryr_k8s_opts)
 CONF.register_opts(daemon_opts, group='cni_daemon')
@@ -311,9 +341,9 @@ CONF.register_opts(k8s_opts, group='kubernetes')
 CONF.register_opts(neutron_defaults, group='neutron_defaults')
 CONF.register_opts(octavia_defaults, group='octavia_defaults')
 CONF.register_opts(cache_defaults, group='cache_defaults')
-CONF.register_opts(ingress, group='ingress')
 CONF.register_opts(nested_vif_driver_opts, group='pod_vif_nested')
 CONF.register_opts(sriov_opts, group='sriov')
+CONF.register_opts(vhostuser, group='vhostuser')
 
 CONF.register_opts(lib_config.core_opts)
 CONF.register_opts(lib_config.binding_opts, 'binding')
@@ -325,6 +355,8 @@ logging.register_options(CONF)
 def init(args, **kwargs):
     version_k8s = version.version_info.version_string()
     CONF(args=args, project='kuryr-k8s', version=version_k8s, **kwargs)
+    if os.environ.get('CNI_COMMAND') == 'VERSION':
+        CONF.set_default('use_stderr', True)
 
 
 def setup_logging():
