@@ -14,7 +14,6 @@
 #    under the License.
 
 import abc
-import six
 
 from kuryr.lib._i18n import _
 from stevedore import driver as stv_driver
@@ -38,8 +37,7 @@ class DriverBase(object):
 
     Usage example:
 
-        @six.add_metaclass(abc.ABCMeta)
-        class SomeDriverInterface(DriverBase):
+        class SomeDriverInterface(DriverBase, metaclass=abc.ABCMeta):
             ALIAS = 'driver_alias'
 
             @abc.abstractmethod
@@ -93,8 +91,7 @@ class DriverBase(object):
         return self.__class__.__name__
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PodProjectDriver(DriverBase):
+class PodProjectDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provides an OpenStack project ID for Kubernetes Pod ports."""
 
     ALIAS = 'pod_project'
@@ -110,8 +107,7 @@ class PodProjectDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ServiceProjectDriver(DriverBase):
+class ServiceProjectDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provides an OpenStack project ID for Kubernetes Services."""
 
     ALIAS = 'service_project'
@@ -127,8 +123,7 @@ class ServiceProjectDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class NamespaceProjectDriver(DriverBase):
+class NamespaceProjectDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provides an OpenStack project ID for Kubernetes Namespace."""
 
     ALIAS = 'namespace_project'
@@ -144,8 +139,7 @@ class NamespaceProjectDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PodSubnetsDriver(DriverBase):
+class PodSubnetsDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provides subnets for Kubernetes Pods."""
 
     ALIAS = 'pod_subnets'
@@ -176,7 +170,7 @@ class PodSubnetsDriver(DriverBase):
     def delete_namespace_subnet(self, kuryr_net_crd):
         """Delete network resources associated to a namespace.
 
-        :param kuryr_net_crd: kuryrnet CRD obj dict that contains Neutron's
+        :param kuryr_net_crd: kuryrnetwork CRD obj dict that contains Neutron's
                               network resources associated to a namespace
         """
         raise NotImplementedError()
@@ -202,8 +196,7 @@ class PodSubnetsDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ServiceSubnetsDriver(DriverBase):
+class ServiceSubnetsDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provides subnets for Kubernetes Services."""
 
     ALIAS = 'service_subnets'
@@ -222,8 +215,7 @@ class ServiceSubnetsDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PodSecurityGroupsDriver(DriverBase):
+class PodSecurityGroupsDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provides security groups for Kubernetes Pods."""
 
     ALIAS = 'pod_security_groups'
@@ -238,26 +230,6 @@ class PodSecurityGroupsDriver(DriverBase):
         """
         raise NotImplementedError()
 
-    def create_namespace_sg(self, namespace, project_id, crd_spec):
-        """Create security group resources for a namespace.
-
-        :param namespace: string with the namespace name
-        :param project_id: OpenStack project ID
-        :param crd_spec: dict with the keys and values for the CRD spec, such
-                         as subnetId or subnetCIDR
-        :return: dict with the keys and values for the CRD spec, such as sgId.
-        If no security group need to be created for the namespace, it
-        should return an empty dict
-        """
-        raise NotImplementedError()
-
-    def delete_sg(self, sg_id):
-        """Delete security group associated to a namespace.
-
-        :param sg_id: OpenStack security group ID
-        """
-        raise NotImplementedError()
-
     def create_sg_rules(self, pod):
         """Create security group rules for a pod.
 
@@ -307,8 +279,7 @@ class PodSecurityGroupsDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ServiceSecurityGroupsDriver(DriverBase):
+class ServiceSecurityGroupsDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provides security groups for Kubernetes Services."""
 
     ALIAS = 'service_security_groups'
@@ -324,8 +295,7 @@ class ServiceSecurityGroupsDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PodVIFDriver(DriverBase):
+class PodVIFDriver(DriverBase, metaclass=abc.ABCMeta):
     """Manages Neutron ports to provide VIFs for Kubernetes Pods."""
 
     ALIAS = 'pod_vif'
@@ -418,7 +388,7 @@ class PodVIFDriver(DriverBase):
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def activate_vif(self, pod, vif):
+    def activate_vif(self, vif):
         """Updates VIF to become active.
 
         Implementing drivers should update the specified `vif` object's
@@ -433,7 +403,6 @@ class PodVIFDriver(DriverBase):
         This method may be called before, after or while the VIF is being
         plugged by the CNI plugin.
 
-        :param pod: dict containing Kubernetes Pod object
         :param vif: VIF object as returned by `PodVIFDriver.request_vif`
         """
         raise NotImplementedError()
@@ -453,8 +422,7 @@ class PodVIFDriver(DriverBase):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class MultiVIFDriver(DriverBase):
+class MultiVIFDriver(DriverBase, metaclass=abc.ABCMeta):
     """Manages additional ports of Kubernetes Pods."""
 
     ALIAS = 'multi_vif'
@@ -631,91 +599,6 @@ class LBaaSDriver(DriverBase):
         """
         raise NotImplementedError()
 
-    @abc.abstractmethod
-    def get_lb_by_uuid(self, lb_uuid):
-        """Get loadbalancer by loadbalancer uuid.
-
-        :param lb_uuid: Loadbalancer uuid
-        """
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def get_pool_by_name(self, pool_name, project_id):
-        """Get pool by pool's name.
-
-        :param pool_name: the pool name
-        :param project_id: project id
-        """
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def ensure_l7_policy(self, namespace, route_name, loadbalancer,
-                         pool, listener_id):
-        """Get or create L7 policy.
-
-        :param namespace: ocp-route/k8s-ingress namespace
-        :param route_name: ocp-route/k8s-ingress name
-        :param loadbalancer: `LBaaSLoadBalancer` object
-        :param pool: L7 policy's target pool
-        :param listener_id: ID of listener to attach L7policy to
-
-        """
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def release_l7_policy(self, loadbalancer, l7_policy):
-        """Release l7policy.
-
-        :param loadbalancer: `LBaaSLoadBalancer` object
-        :param l7_policy: `LBaaSL7Policy` object
-
-        """
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def ensure_l7_rule(self, loadbalancer, l7_policy, compare_type,
-                       type, value):
-        """Get or create L7 rule.
-
-        :param loadbalancer: `LBaaSLoadBalancer` object
-        :param l7_policy: `LBaaSL7Policy` object
-        :param compare_type: comparison type for the L7 rule.
-        :param type: the L7 rule type
-        :param value:the value to use for the comparison.
-
-        """
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def release_l7_rule(self, loadbalancer, l7_rule):
-        """Release L7 rule.
-
-        :param loadbalancer: `LBaaSLoadBalancer` object
-        :param l7_rule: `LBaaSL7Rule` object
-
-        """
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def update_l7_rule(self, l7_rule, new_value):
-        """Update L7 rule value.
-
-        :param l7_rule: `LBaaSL7Rule` object
-        :param new_value: rule's new value
-
-        """
-        raise NotImplementedError()
-
-    @abc.abstractmethod
-    def is_pool_used_by_other_l7policies(self, l7policy, pool):
-        """Checks if pool used by other L7policy.
-
-        :param l7policy: `LBaaSL7Policy` object
-        :param pool: `LBaaSPool` object
-
-        """
-        raise NotImplementedError()
-
     @abc.abstractmethod
     def update_lbaas_sg(self, service, sgs):
         """Update security group rules associated to the loadbalancer
@@ -725,9 +608,13 @@ class LBaaSDriver(DriverBase):
         """
         raise NotImplementedError()
 
+    @abc.abstractmethod
+    def add_tags(self, resource, req):
+        """Add tags to a request if the resource supports it"""
+        raise NotImplementedError()
 
-@six.add_metaclass(abc.ABCMeta)
-class VIFPoolDriver(PodVIFDriver):
+
+class VIFPoolDriver(PodVIFDriver, metaclass=abc.ABCMeta):
     """Manages Pool of Neutron ports to provide VIFs for Kubernetes Pods."""
 
     ALIAS = 'vif_pool'
@@ -757,8 +644,7 @@ class VIFPoolDriver(PodVIFDriver):
         raise NotImplementedError()
 
 
-@six.add_metaclass(abc.ABCMeta)
-class ServicePubIpDriver(DriverBase):
+class ServicePubIpDriver(DriverBase, metaclass=abc.ABCMeta):
     """Manages loadbalancerIP/public ip for neutron lbaas."""
 
     ALIAS = 'service_public_ip'
@@ -805,20 +691,16 @@ class ServicePubIpDriver(DriverBase):
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class NetworkPolicyDriver(DriverBase):
+class NetworkPolicyDriver(DriverBase, metaclass=abc.ABCMeta):
     """Provide network-policy for pods"""
 
     ALIAS = 'network_policy'
 
     @abc.abstractmethod
-    def ensure_network_policy(self, policy, project_id):
+    def ensure_network_policy(self, policy):
         """Policy created or updated
 
         :param policy: dict containing Kubernetes NP object
-        :param project_id: openstack project_id
-        :returns: list of Pod objects affected by the network policy
-        creation or its podSelector modification
         """
         raise NotImplementedError()
 
@@ -826,7 +708,7 @@ class NetworkPolicyDriver(DriverBase):
     def release_network_policy(self, kuryrnetpolicy):
         """Delete a network policy
 
-        :param kuryrnetpolicy: dict containing Kuryrnetpolicy CRD object
+        :param kuryrnetpolicy: dict containing NetworkPolicy object
         """
         raise NotImplementedError()
 
@@ -844,18 +726,6 @@ class NetworkPolicyDriver(DriverBase):
         """
         raise NotImplementedError()
 
-    @abc.abstractmethod
-    def knps_on_namespace(self, namespace):
-        """Check if there si kuryr network policy CRDs on the namespace
-
-        This method returns true if there are knps on the specified namespace
-        or false otherwise
-
-        :param namespace: namespace name where the knps CRDs should be
-        :returns: true if knps CRDs on the namespace, false otherwise
-        """
-        raise NotImplementedError()
-
     @abc.abstractmethod
     def namespaced_pods(self, policy):
         """Return pods on the policy namespace
@@ -867,18 +737,8 @@ class NetworkPolicyDriver(DriverBase):
         """
         raise NotImplementedError()
 
-    @abc.abstractmethod
-    def get_kuryrnetpolicy_crd(self, policy):
-        """Return kuryrnetpolicy CRD object associated to the policy
 
-        :param policy: dict containing Kubernetes NP object
-        :returns: kuryrnetpolicy CRD object associated to the policy
-        """
-        raise NotImplementedError()
-
-
-@six.add_metaclass(abc.ABCMeta)
-class NetworkPolicyProjectDriver(DriverBase):
+class NetworkPolicyProjectDriver(DriverBase, metaclass=abc.ABCMeta):
     """Get an OpenStack project id for K8s network policies"""
 
     ALIAS = 'network_policy_project'
