@@ -15,7 +15,9 @@
 
 import urllib
 
+import netaddr
 from openstack import exceptions as os_exc
+from os_vif import objects
 from oslo_config import cfg
 from oslo_log import log
 from oslo_serialization import jsonutils
@@ -58,19 +60,27 @@ def get_host_id(pod):
     return pod['spec']['nodeName']
 
 
-def get_pod_state(pod):
+def get_kuryrport(pod):
+    k8s = clients.get_kubernetes_client()
     try:
-        annotations = pod['metadata']['annotations']
-        state_annotation = annotations[constants.K8S_ANNOTATION_VIF]
-    except KeyError:
+        return k8s.get(f'{constants.K8S_API_CRD_NAMESPACES}/'
+                       f'{pod["metadata"]["namespace"]}/kuryrports/'
+                       f'{pod["metadata"]["name"]}')
+    except k_exc.K8sResourceNotFound:
         return None
-    state_annotation = jsonutils.loads(state_annotation)
-    state = utils.extract_pod_annotation(state_annotation)
-    return state
+
+
+def get_vifs(pod):
+    kp = get_kuryrport(pod)
+    try:
+        return {k: objects.base.VersionedObject.obj_from_primitive(v['vif'])
+                for k, v in kp['status']['vifs'].items()}
+    except (KeyError, AttributeError, TypeError):
+        return {}
 
 
 def is_host_network(pod):
-    return pod['spec'].get('hostNetwork', False)
+    return pod['status'].get('hostNetwork', False)
 
 
 def get_pods(selector, namespace=None):
@@ -173,10 +183,8 @@ def replace_encoded_characters(labels):
 def create_security_group_rule(body):
     os_net = clients.get_network_client()
 
-    sgr = ''
-
     try:
-        params = dict(body['security_group_rule'])
+        params = dict(body)
         if 'ethertype' in params:
             # NOTE(gryf): in openstacksdk, there is ether_type attribute in
             # the security_group_rule object, in CRD we have 'ethertype'
@@ -186,10 +194,15 @@ def create_security_group_rule(body):
         sgr = os_net.create_security_group_rule(**params)
         return sgr.id
     except os_exc.ConflictException as ex:
-        LOG.debug("Failed to create already existing security group "
-                  "rule %s", body)
-        # Get existent sg rule id from exception message
-        return str(ex).split()[-1][:-1]
+        if 'quota' in ex.details.lower():
+            LOG.error("Failed to create security group rule %s: %s", body,
+                      ex.details)
+            raise
+        else:
+            LOG.debug("Failed to create already existing security group "
+                      "rule %s", body)
+            # Get existent sg rule id from exception message
+            return str(ex).split()[-1][:-1]
     except os_exc.SDKException:
         LOG.debug("Error creating security group rule")
         raise
@@ -206,29 +219,27 @@ def delete_security_group_rule(security_group_rule_id):
         raise
 
 
-def patch_kuryrnetworkpolicy_crd(crd, i_rules, e_rules, pod_selector,
-                                 np_spec=None):
+def patch_kuryrnetworkpolicy_crd(crd, i_rules, e_rules):
     kubernetes = clients.get_kubernetes_client()
     crd_name = crd['metadata']['name']
-    if not np_spec:
-        np_spec = crd['spec']['networkpolicy_spec']
-    LOG.debug('Patching KuryrNetPolicy CRD %s' % crd_name)
+    LOG.debug('Patching KuryrNetworkPolicy CRD %s' % crd_name)
     try:
-        kubernetes.patch_crd('spec', crd['metadata']['selfLink'],
-                             {'ingressSgRules': i_rules,
-                              'egressSgRules': e_rules,
-                              'podSelector': pod_selector,
-                              'networkpolicy_spec': np_spec})
+        spec = {
+            'ingressSgRules': i_rules,
+            'egressSgRules': e_rules,
+        }
+
+        kubernetes.patch_crd('spec', crd['metadata']['selfLink'], spec)
     except k_exc.K8sResourceNotFound:
-        LOG.debug('KuryrNetPolicy CRD not found %s', crd_name)
+        LOG.debug('KuryrNetworkPolicy CRD not found %s', crd_name)
     except k_exc.K8sClientException:
-        LOG.exception('Error updating kuryrnetpolicy CRD %s', crd_name)
+        LOG.exception('Error updating KuryrNetworkPolicy CRD %s', crd_name)
         raise
 
 
 def create_security_group_rule_body(
-        security_group_id, direction, port_range_min=None,
-        port_range_max=None, protocol=None, ethertype='IPv4', cidr=None,
+        direction, port_range_min=None, port_range_max=None, protocol=None,
+        ethertype='IPv4', cidr=None,
         description="Kuryr-Kubernetes NetPolicy SG rule", namespace=None,
         pods=None):
     if not port_range_min:
@@ -238,10 +249,13 @@ def create_security_group_rule_body(
         port_range_max = port_range_min
     if not protocol:
         protocol = 'TCP'
+
+    if cidr and netaddr.IPNetwork(cidr).version == 6:
+        ethertype = 'IPv6'
+
     security_group_rule_body = {
-        'security_group_rule': {
+        'sgRule': {
             'ethertype': ethertype,
-            'security_group_id': security_group_id,
             'description': description,
             'direction': direction,
             'protocol': protocol.lower(),
@@ -250,31 +264,29 @@ def create_security_group_rule_body(
         }
     }
     if cidr:
-        security_group_rule_body['security_group_rule'][
-            'remote_ip_prefix'] = cidr
+        security_group_rule_body['sgRule']['remote_ip_prefix'] = cidr
     if namespace:
         security_group_rule_body['namespace'] = namespace
     if pods:
-        security_group_rule_body['remote_ip_prefixes'] = pods
+        security_group_rule_body['affectedPods'] = [
+            {'podIP': ip, 'podNamespace': ns} for ip, ns in pods.items() if ip]
     LOG.debug("Creating sg rule body %s", security_group_rule_body)
     return security_group_rule_body
 
 
 def get_pod_ip(pod):
     try:
-        pod_metadata = pod['metadata']['annotations']
-        vif = pod_metadata[constants.K8S_ANNOTATION_VIF]
-    except KeyError:
+        kp = get_kuryrport(pod)
+        vif = [x['vif'] for x in kp['status']['vifs'].values()
+               if x['default']][0]
+    except (KeyError, TypeError, IndexError):
         return None
-    vif = jsonutils.loads(vif)
-    vif = vif['versioned_object.data']['default_vif']
-    network = (vif['versioned_object.data']['network']
-                  ['versioned_object.data'])
-    first_subnet = (network['subnets']['versioned_object.data']
-                    ['objects'][0]['versioned_object.data'])
-    first_subnet_ip = (first_subnet['ips']['versioned_object.data']
-                       ['objects'][0]['versioned_object.data']['address'])
-    return first_subnet_ip
+    return (vif['versioned_object.data']['network']
+            ['versioned_object.data']['subnets']
+            ['versioned_object.data']['objects'][0]
+            ['versioned_object.data']['ips']
+            ['versioned_object.data']['objects'][0]
+            ['versioned_object.data']['address'])
 
 
 def get_annotations(resource, annotation):
@@ -292,25 +304,60 @@ def get_annotated_labels(resource, annotation_labels):
     return None
 
 
-def get_kuryrnetpolicy_crds(namespace=None):
+def get_kuryrnetworkpolicy_crds(namespace=None):
     kubernetes = clients.get_kubernetes_client()
 
     try:
         if namespace:
-            knp_path = '{}/{}/kuryrnetpolicies'.format(
+            knp_path = '{}/{}/kuryrnetworkpolicies'.format(
                 constants.K8S_API_CRD_NAMESPACES, namespace)
         else:
-            knp_path = constants.K8S_API_CRD_KURYRNETPOLICIES
-        LOG.debug("K8s API Query %s", knp_path)
+            knp_path = constants.K8S_API_CRD_KURYRNETWORKPOLICIES
         knps = kubernetes.get(knp_path)
-        LOG.debug("Return Kuryr Network Policies with label %s", knps)
+        LOG.debug("Returning KuryrNetworkPolicies %s", knps)
     except k_exc.K8sResourceNotFound:
-        LOG.exception("KuryrNetPolicy CRD not found")
-        raise
+        LOG.exception("KuryrNetworkPolicy CRD not found")
+        return []
     except k_exc.K8sClientException:
         LOG.exception("Kubernetes Client Exception")
         raise
-    return knps
+    return knps.get('items', [])
+
+
+def get_networkpolicies(namespace=None):
+    # FIXME(dulek): This is awful, shouldn't we have list method on k8s_client?
+    kubernetes = clients.get_kubernetes_client()
+
+    try:
+        if namespace:
+            np_path = '{}/{}/networkpolicies'.format(
+                constants.K8S_API_CRD_NAMESPACES, namespace)
+        else:
+            np_path = constants.K8S_API_POLICIES
+        nps = kubernetes.get(np_path)
+    except k_exc.K8sResourceNotFound:
+        LOG.exception("NetworkPolicy or namespace %s not found", namespace)
+        raise
+    except k_exc.K8sClientException:
+        LOG.exception("Exception when listing NetworkPolicies.")
+        raise
+    return nps.get('items', [])
+
+
+def zip_knp_np(knps, nps):
+    """Returns tuples of matching KuryrNetworkPolicy and NetworkPolicy objs.
+
+    :param knps: List of KuryrNetworkPolicy objects
+    :param nps: List of NetworkPolicy objects
+    :return: List of tuples of matching (knp, np)
+    """
+    pairs = []
+    for knp in knps:
+        for np in nps:
+            if utils.get_res_unique_name(knp) == utils.get_res_unique_name(np):
+                pairs.append((knp, np))
+                break
+    return pairs
 
 
 def match_expressions(expressions, labels):
@@ -353,15 +400,14 @@ def match_labels(crd_labels, labels):
 def match_selector(selector, labels):
     if selector is None:
         return True
-    if labels is None:
-        labels = {}
     crd_labels = selector.get('matchLabels', None)
     crd_expressions = selector.get('matchExpressions', None)
 
     match_exp = match_lb = True
     if crd_expressions:
-        match_exp = match_expressions(crd_expressions, labels)
-    if crd_labels:
+        match_exp = match_expressions(crd_expressions,
+                                      labels)
+    if crd_labels and labels:
         match_lb = match_labels(crd_labels, labels)
     return match_exp and match_lb
 
