@@ -16,15 +16,19 @@ import contextlib
 import itertools
 import os
 import ssl
+import time
+from urllib import parse
 
 from oslo_log import log as logging
 from oslo_serialization import jsonutils
 import requests
+from requests import adapters
 
 from kuryr.lib._i18n import _
 from kuryr_kubernetes import config
 from kuryr_kubernetes import constants
 from kuryr_kubernetes import exceptions as exc
+from kuryr_kubernetes import utils
 
 CONF = config.CONF
 LOG = logging.getLogger(__name__)
@@ -43,6 +47,12 @@ class K8sClient(object):
         token_file = config.CONF.kubernetes.token_file
         self.token = None
         self.cert = (None, None)
+
+        # Setting higher numbers regarding connection pools as we're running
+        # with max of 1000 green threads.
+        self.session = requests.Session()
+        prefix = '%s://' % parse.urlparse(base_url).scheme
+        self.session.mount(prefix, adapters.HTTPAdapter(pool_maxsize=1000))
         if token_file:
             if os.path.exists(token_file):
                 with open(token_file, 'r') as f:
@@ -68,6 +78,26 @@ class K8sClient(object):
             else:
                 self.verify_server = ca_crt_file
 
+    def _raise_from_response(self, response):
+        if response.status_code == requests.codes.not_found:
+            raise exc.K8sResourceNotFound(response.text)
+        if response.status_code == requests.codes.conflict:
+            raise exc.K8sConflict(response.text)
+        if response.status_code == requests.codes.forbidden:
+            if 'because it is being terminated' in response.json()['message']:
+                raise exc.K8sNamespaceTerminating(response.text)
+            raise exc.K8sForbidden(response.text)
+        if response.status_code == requests.codes.unprocessable_entity:
+            # NOTE(gryf): on k8s API code 422 is also Forbidden, but specified
+            # to FieldValueForbidden. Perhaps there are other usages for
+            # throwing unprocessable entity errors in different cases.
+            if ('FieldValueForbidden' in response.text and
+                    'Forbidden' in response.json()['message']):
+                raise exc.K8sFieldValueForbidden(response.text)
+            raise exc.K8sUnprocessableEntity(response.text)
+        if not response.ok:
+            raise exc.K8sClientException(response.text)
+
     def get(self, path, json=True, headers=None):
         LOG.debug("Get %(path)s", {'path': path})
         url = self._base_url + path
@@ -76,13 +106,10 @@ class K8sClient(object):
             header.update({'Authorization': 'Bearer %s' % self.token})
         if headers:
             header.update(headers)
-        response = requests.get(url, cert=self.cert,
-                                verify=self.verify_server,
-                                headers=header)
-        if response.status_code == requests.codes.not_found:
-            raise exc.K8sResourceNotFound(response.text)
-        if not response.ok:
-            raise exc.K8sClientException(response.text)
+        response = self.session.get(url, cert=self.cert,
+                                    verify=self.verify_server,
+                                    headers=header)
+        self._raise_from_response(response)
         result = response.json() if json else response.text
         return result
 
@@ -98,35 +125,40 @@ class K8sClient(object):
     def patch(self, field, path, data):
         LOG.debug("Patch %(path)s: %(data)s", {
             'path': path, 'data': data})
-        if field == 'status':
-            path = path + '/' + str(field)
         content_type = 'application/merge-patch+json'
         url, header = self._get_url_and_header(path, content_type)
-        response = requests.patch(url, json={field: data},
-                                  headers=header, cert=self.cert,
-                                  verify=self.verify_server)
-        if response.ok:
-            return response.json().get('status')
-        raise exc.K8sClientException(response.text)
+        response = self.session.patch(url, json={field: data},
+                                      headers=header, cert=self.cert,
+                                      verify=self.verify_server)
+        self._raise_from_response(response)
+        return response.json().get('status')
 
-    def patch_crd(self, field, path, data):
+    def patch_crd(self, field, path, data, action='replace'):
         content_type = 'application/json-patch+json'
         url, header = self._get_url_and_header(path, content_type)
 
-        data = [{'op': 'replace',
-                 'path': '/{}/{}'.format(field, np_field),
-                 'value': value}
-                for np_field, value in data.items()]
+        if action == 'remove':
+            data = [{'op': action,
+                     'path': f'/{field}/{data}'}]
+        else:
+            if data:
+                data = [{'op': action,
+                         'path': f'/{field}/{crd_field}',
+                         'value': value}
+                        for crd_field, value in data.items()]
+            else:
+                data = [{'op': action,
+                         'path': f'/{field}',
+                         'value': data}]
 
         LOG.debug("Patch %(path)s: %(data)s", {
             'path': path, 'data': data})
 
-        response = requests.patch(url, data=jsonutils.dumps(data),
-                                  headers=header, cert=self.cert,
-                                  verify=self.verify_server)
-        if response.ok:
-            return response.json().get('status')
-        raise exc.K8sClientException(response.text)
+        response = self.session.patch(url, data=jsonutils.dumps(data),
+                                      headers=header, cert=self.cert,
+                                      verify=self.verify_server)
+        self._raise_from_response(response)
+        return response.json().get('status')
 
     def patch_node_annotations(self, node, annotation_name, value):
         content_type = 'application/json-patch+json'
@@ -138,12 +170,11 @@ class K8sClient(object):
                  'path': '/metadata/annotations/{}'.format(annotation_name),
                  'value': value}]
 
-        response = requests.patch(url, data=jsonutils.dumps(data),
-                                  headers=header, cert=self.cert,
-                                  verify=self.verify_server)
-        if response.ok:
-            return response.json().get('status')
-        raise exc.K8sClientException(response.text)
+        response = self.session.patch(url, data=jsonutils.dumps(data),
+                                      headers=header, cert=self.cert,
+                                      verify=self.verify_server)
+        self._raise_from_response(response)
+        return response.json().get('status')
 
     def remove_node_annotations(self, node, annotation_name):
         content_type = 'application/json-patch+json'
@@ -153,9 +184,29 @@ class K8sClient(object):
         data = [{'op': 'remove',
                  'path': '/metadata/annotations/{}'.format(annotation_name)}]
 
-        response = requests.patch(url, data=jsonutils.dumps(data),
-                                  headers=header, cert=self.cert,
-                                  verify=self.verify_server)
+        response = self.session.patch(url, data=jsonutils.dumps(data),
+                                      headers=header, cert=self.cert,
+                                      verify=self.verify_server)
+        self._raise_from_response(response)
+        return response.json().get('status')
+
+    def _jsonpatch_escape(self, value):
+        value = value.replace('~', '~0')
+        value = value.replace('/', '~1')
+        return value
+
+    def remove_annotations(self, path, annotation_name):
+        LOG.debug("Remove annotations %(path)s: %(name)s",
+                  {'path': path, 'name': annotation_name})
+        content_type = 'application/json-patch+json'
+        url, header = self._get_url_and_header(path, content_type)
+        annotation_name = self._jsonpatch_escape(annotation_name)
+
+        data = [{'op': 'remove',
+                 'path': f'/metadata/annotations/{annotation_name}'}]
+        response = self.session.patch(url, data=jsonutils.dumps(data),
+                                      headers=header, cert=self.cert,
+                                      verify=self.verify_server)
         if response.ok:
             return response.json().get('status')
         raise exc.K8sClientException(response.text)
@@ -167,11 +218,10 @@ class K8sClient(object):
         if self.token:
             header.update({'Authorization': 'Bearer %s' % self.token})
 
-        response = requests.post(url, json=body, cert=self.cert,
-                                 verify=self.verify_server, headers=header)
-        if response.ok:
-            return response.json()
-        raise exc.K8sClientException(response)
+        response = self.session.post(url, json=body, cert=self.cert,
+                                     verify=self.verify_server, headers=header)
+        self._raise_from_response(response)
+        return response.json()
 
     def delete(self, path):
         LOG.debug("Delete %(path)s", {'path': path})
@@ -180,14 +230,109 @@ class K8sClient(object):
         if self.token:
             header.update({'Authorization': 'Bearer %s' % self.token})
 
-        response = requests.delete(url, cert=self.cert,
-                                   verify=self.verify_server, headers=header)
-        if response.ok:
-            return response.json()
-        else:
-            if response.status_code == requests.codes.not_found:
-                raise exc.K8sResourceNotFound(response.text)
-            raise exc.K8sClientException(response)
+        response = self.session.delete(url, cert=self.cert,
+                                       verify=self.verify_server,
+                                       headers=header)
+        self._raise_from_response(response)
+        return response.json()
+
+    # TODO(dulek): add_finalizer() and remove_finalizer() have some code
+    #              duplication, but I don't see a nice way to avoid it.
+    def add_finalizer(self, obj, finalizer):
+        if finalizer in obj['metadata'].get('finalizers', []):
+            return True
+
+        path = obj['metadata']['selfLink']
+        LOG.debug(f"Add finalizer {finalizer} to {path}")
+        url, headers = self._get_url_and_header(
+            path, 'application/merge-patch+json')
+
+        for i in range(3):  # Let's make sure it's not infinite loop
+            finalizers = obj['metadata'].get('finalizers', []).copy()
+            finalizers.append(finalizer)
+
+            data = {
+                'metadata': {
+                    'finalizers': finalizers,
+                    'resourceVersion': obj['metadata']['resourceVersion'],
+                },
+            }
+
+            response = self.session.patch(url, json=data, headers=headers,
+                                          cert=self.cert,
+                                          verify=self.verify_server)
+
+            if response.ok:
+                return True
+
+            try:
+                self._raise_from_response(response)
+            except (exc.K8sFieldValueForbidden, exc.K8sResourceNotFound):
+                # Object is being deleting or gone. Return.
+                return False
+            except exc.K8sConflict:
+                obj = self.get(path)
+                if finalizer in obj['metadata'].get('finalizers', []):
+                    # Finalizer is there, return.
+                    return True
+
+        # If after 3 iterations there's still conflict, just raise.
+        self._raise_from_response(response)
+
+    def remove_finalizer(self, obj, finalizer):
+        path = obj['metadata']['selfLink']
+        LOG.debug(f"Remove finalizer {finalizer} from {path}")
+        url, headers = self._get_url_and_header(
+            path, 'application/merge-patch+json')
+
+        for i in range(3):  # Let's make sure it's not infinite loop
+            finalizers = obj['metadata'].get('finalizers', []).copy()
+            try:
+                finalizers.remove(finalizer)
+            except ValueError:
+                # Finalizer is not there, return.
+                return True
+
+            data = {
+                'metadata': {
+                    'finalizers': finalizers,
+                    'resourceVersion': obj['metadata']['resourceVersion'],
+                },
+            }
+
+            response = self.session.patch(url, json=data, headers=headers,
+                                          cert=self.cert,
+                                          verify=self.verify_server)
+
+            if response.ok:
+                return True
+
+            try:
+                try:
+                    self._raise_from_response(response)
+                except exc.K8sConflict:
+                    obj = self.get(path)
+            except (exc.K8sFieldValueForbidden, exc.K8sResourceNotFound):
+                # Object is being deleted or gone already, stop.
+                return False
+
+        # If after 3 iterations there's still conflict, just raise.
+        self._raise_from_response(response)
+
+    def get_loadbalancer_crd(self, obj):
+        name = obj['metadata']['name']
+        namespace = obj['metadata']['namespace']
+
+        try:
+            crd = self.get('{}/{}/kuryrloadbalancers/{}'.format(
+                constants.K8S_API_CRD_NAMESPACES, namespace,
+                name))
+        except exc.K8sResourceNotFound:
+            return None
+        except exc.K8sClientException:
+            LOG.exception("Kubernetes Client Exception.")
+            raise
+        return crd
 
     def annotate(self, path, annotations, resource_version=None):
         """Pushes a resource annotation to the K8s API resource
@@ -195,7 +340,7 @@ class K8sClient(object):
         The annotate operation is made with a PATCH HTTP request of kind:
         application/merge-patch+json as described in:
 
-        https://github.com/kubernetes/community/blob/master/contributors/devel/api-conventions.md#patch-operations  # noqa
+        https://github.com/kubernetes/community/blob/master/contributors/devel/sig-architecture/api-conventions.md#patch-operations  # noqa
         """
         LOG.debug("Annotate %(path)s: %(names)s", {
             'path': path, 'names': list(annotations)})
@@ -208,11 +353,11 @@ class K8sClient(object):
             if resource_version:
                 metadata['resourceVersion'] = resource_version
             data = jsonutils.dumps({"metadata": metadata}, sort_keys=True)
-            response = requests.patch(url, data=data,
-                                      headers=header, cert=self.cert,
-                                      verify=self.verify_server)
+            response = self.session.patch(url, data=data,
+                                          headers=header, cert=self.cert,
+                                          verify=self.verify_server)
             if response.ok:
-                return response.json()['metadata']['annotations']
+                return response.json()['metadata'].get('annotations', {})
             if response.status_code == requests.codes.conflict:
                 resource = self.get(path)
                 new_version = resource['metadata']['resourceVersion']
@@ -236,10 +381,7 @@ class K8sClient(object):
                       % {'headers': response.headers,
                          'content': response.content, 'text': response.text})
 
-            if response.status_code == requests.codes.not_found:
-                raise exc.K8sResourceNotFound(response.text)
-            else:
-                raise exc.K8sClientException(response.text)
+            self._raise_from_response(response)
 
     def watch(self, path):
         url = self._base_url + path
@@ -250,18 +392,20 @@ class K8sClient(object):
         if self.token:
             header.update({'Authorization': 'Bearer %s' % self.token})
 
+        attempt = 0
         while True:
             try:
                 params = {'watch': 'true'}
                 if resource_version:
                     params['resourceVersion'] = resource_version
                 with contextlib.closing(
-                        requests.get(
+                        self.session.get(
                             url, params=params, stream=True, cert=self.cert,
                             verify=self.verify_server, headers=header,
                             timeout=timeouts)) as response:
                     if not response.ok:
                         raise exc.K8sClientException(response.text)
+                    attempt = 0
                     for line in response.iter_lines():
                         line = line.decode('utf-8').strip()
                         if line:
@@ -271,19 +415,16 @@ class K8sClient(object):
                             # At this point it's safely passed to handler.
                             m = line_dict.get('object', {}).get('metadata', {})
                             resource_version = m.get('resourceVersion', None)
-            except (requests.ReadTimeout, ssl.SSLError) as e:
-                if isinstance(e, ssl.SSLError) and e.args != ('timed out',):
-                    raise
-
-                LOG.warning('%ds without data received from watching %s. '
-                            'Retrying the connection with resourceVersion=%s.',
-                            timeouts[1], path, params.get('resourceVersion'))
-            except requests.exceptions.ChunkedEncodingError:
-                LOG.warning("Connection to %s closed when watching. This "
-                            "mostly happens when Octavia's Amphora closes "
-                            "connection due to lack of activity for 50s. "
-                            "Since Rocky Octavia this is configurable and "
-                            "should be set to at least 20m, so check timeouts "
-                            "on Kubernetes API LB listener. Restarting "
-                            "connection with resourceVersion=%s.", path,
-                            params.get('resourceVersion'))
+            except (requests.ReadTimeout, requests.ConnectionError,
+                    ssl.SSLError, requests.exceptions.ChunkedEncodingError):
+                t = utils.exponential_backoff(attempt)
+                log = LOG.debug
+                if attempt > 0:
+                    # Only make it a warning if it's happening again, no need
+                    # to inform about all the read timeouts.
+                    log = LOG.warning
+                log('Connection error when watching %s. Retrying in %ds with '
+                    'resourceVersion=%s', path, t,
+                    params.get('resourceVersion'))
+                time.sleep(t)
+                attempt += 1
