@@ -13,12 +13,15 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from unittest import mock
+
 import fixtures
-import mock
 import time
 
+from kuryr_kubernetes import exceptions
 from kuryr_kubernetes.handlers import retry as h_retry
 from kuryr_kubernetes.tests import base as test_base
+from kuryr_kubernetes.tests.unit import kuryr_fixtures as k_fix
 
 
 class _EX1(Exception):
@@ -42,9 +45,13 @@ class TestRetryHandler(test_base.TestCase):
         f_time = self.useFixture(fixtures.MockPatch('time.time'))
         f_time.mock.return_value = self.now
 
-    @mock.patch('random.randint')
+        self.k8s = self.useFixture(k_fix.MockK8sClient()).client
+        f_k8s = self.useFixture(fixtures.MockPatch(
+            'kuryr_kubernetes.clients.get_kubernetes_client'))
+        f_k8s.mock.return_value = self.k8s
+
     @mock.patch('time.sleep')
-    def test_should_not_sleep(self, m_sleep, m_randint):
+    def test_should_not_sleep(self, m_sleep):
         deadline = self.now - 1
         retry = h_retry.Retry(mock.Mock())
 
@@ -52,28 +59,25 @@ class TestRetryHandler(test_base.TestCase):
 
         self.assertFalse(ret)
         m_sleep.assert_not_called()
-        m_randint.assert_not_called()
 
     def _test_should_sleep(self, seconds_left, slept):
-        attempt = 5
+        attempt = 2
         timeout = 20
         interval = 3
-        randint = 2
         deadline = self.now + seconds_left
         retry = h_retry.Retry(mock.Mock(), timeout=timeout, interval=interval)
 
         with mock.patch('random.randint') as m_randint, \
                 mock.patch('time.sleep') as m_sleep:
-            m_randint.return_value = randint
+            m_randint.return_value = 0  # Assume 0 as jitter
 
             ret = retry._sleep(deadline, attempt, _EX2())
 
             self.assertEqual(slept, ret)
-            m_randint.assert_called_once_with(1, 2 ** attempt - 1)
             m_sleep.assert_called_once_with(slept)
 
     def test_should_sleep(self):
-        self._test_should_sleep(7, 6)
+        self._test_should_sleep(20, 12)
 
     def test_should_sleep_last(self):
         self._test_should_sleep(5, 5)
@@ -87,10 +91,27 @@ class TestRetryHandler(test_base.TestCase):
         m_handler = mock.Mock()
         m_count.return_value = list(range(1, 5))
         retry = h_retry.Retry(m_handler)
+        event = {'type': 'DELETED'}
 
-        retry(mock.sentinel.event)
+        retry(event)
 
-        m_handler.assert_called_once_with(mock.sentinel.event)
+        m_handler.assert_called_once_with(event)
+        m_sleep.assert_not_called()
+
+    @mock.patch('itertools.count')
+    @mock.patch.object(h_retry.Retry, '_sleep')
+    def test_call_outdated_event(self, m_sleep, m_count):
+        m_handler = mock.Mock()
+        m_count.return_value = list(range(1, 5))
+        obj = {'metadata': {'selfLink': mock.sentinel.selflink}}
+        event = {'type': 'MODIFIED', 'object': obj}
+        self.k8s.get.side_effect = exceptions.K8sResourceNotFound(obj)
+
+        retry = h_retry.Retry(m_handler)
+        retry(event)
+
+        self.k8s.get.assert_called_once_with(obj['metadata']['selfLink'])
+        m_handler.assert_not_called()
         m_sleep.assert_not_called()
 
     @mock.patch('itertools.count')
@@ -100,7 +121,7 @@ class TestRetryHandler(test_base.TestCase):
         timeout = 10
         deadline = self.now + timeout
         failures = [_EX1()] * (attempts - 1)
-        event = mock.sentinel.event
+        event = {'type': 'DELETED'}
         m_handler = mock.Mock()
         m_handler.side_effect = failures + [None]
         m_sleep.return_value = 1
@@ -121,7 +142,7 @@ class TestRetryHandler(test_base.TestCase):
         timeout = 10
         deadline = self.now + timeout
         failures = [_EX1(), _EX1(), _EX11()]
-        event = mock.sentinel.event
+        event = {'type': 'DELETED'}
         m_handler = mock.Mock()
         m_handler.side_effect = failures
         m_sleep.side_effect = [1] * (attempts - 1) + [0]
