@@ -13,9 +13,9 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import itertools
-import mock
 import os
 import tempfile
+from unittest import mock
 
 from oslo_serialization import jsonutils
 import requests
@@ -26,9 +26,15 @@ from kuryr_kubernetes.tests import base as test_base
 
 
 class TestK8sClient(test_base.TestCase):
-    def setUp(self):
+    @mock.patch('kuryr_kubernetes.config.CONF')
+    def setUp(self, m_cfg):
         super(TestK8sClient, self).setUp()
         self.base_url = 'http://127.0.0.1:12345'
+        m_cfg.kubernetes.ssl_client_crt_file = None
+        m_cfg.kubernetes.ssl_client_key_file = None
+        m_cfg.kubernetes.ssl_ca_crt_file = None
+        m_cfg.kubernetes.token_file = None
+        m_cfg.kubernetes.ssl_verify_server_crt = False
         self.client = k8s_client.K8sClient(self.base_url)
         default_cert = (None, None)
         default_token = None
@@ -68,7 +74,7 @@ class TestK8sClient(test_base.TestCase):
         m_exist.return_value = True
         self.assertRaises(RuntimeError, k8s_client.K8sClient, self.base_url)
 
-    @mock.patch('requests.get')
+    @mock.patch('requests.sessions.Session.get')
     @mock.patch('kuryr_kubernetes.config.CONF')
     def test_bearer_token(self, m_cfg, m_get):
         token_content = (
@@ -104,7 +110,7 @@ class TestK8sClient(test_base.TestCase):
         finally:
             os.unlink(m_cfg.kubernetes.token_file)
 
-    @mock.patch('requests.get')
+    @mock.patch('requests.sessions.Session.get')
     def test_get(self, m_get):
         path = '/test'
         ret = {'test': 'value'}
@@ -119,7 +125,7 @@ class TestK8sClient(test_base.TestCase):
             self.base_url + path,
             cert=(None, None), headers={}, verify=False)
 
-    @mock.patch('requests.get')
+    @mock.patch('requests.sessions.Session.get')
     def test_get_exception(self, m_get):
         path = '/test'
 
@@ -130,7 +136,7 @@ class TestK8sClient(test_base.TestCase):
         self.assertRaises(exc.K8sClientException, self.client.get, path)
 
     @mock.patch('itertools.count')
-    @mock.patch('requests.patch')
+    @mock.patch('requests.sessions.Session.patch')
     def test_annotate(self, m_patch, m_count):
         m_count.return_value = list(range(1, 5))
         path = '/test'
@@ -152,7 +158,7 @@ class TestK8sClient(test_base.TestCase):
                                         cert=(None, None), verify=False)
 
     @mock.patch('itertools.count')
-    @mock.patch('requests.patch')
+    @mock.patch('requests.sessions.Session.patch')
     def test_annotate_exception(self, m_patch, m_count):
         m_count.return_value = list(range(1, 5))
         path = '/test'
@@ -165,7 +171,7 @@ class TestK8sClient(test_base.TestCase):
                           path, {})
 
     @mock.patch('itertools.count')
-    @mock.patch('requests.patch')
+    @mock.patch('requests.sessions.Session.patch')
     def test_annotate_diff_resource_vers_no_conflict(self, m_patch, m_count):
         m_count.return_value = list(range(1, 5))
         path = '/test'
@@ -179,7 +185,6 @@ class TestK8sClient(test_base.TestCase):
             'annotations': annotations,
             'resourceVersion': new_resource_version}}
         conflicting_data = jsonutils.dumps(conflicting_obj, sort_keys=True)
-        good_data = jsonutils.dumps(good_obj, sort_keys=True)
 
         m_resp_conflict = mock.MagicMock()
         m_resp_conflict.ok = False
@@ -198,14 +203,10 @@ class TestK8sClient(test_base.TestCase):
             mock.call(self.base_url + path,
                       data=conflicting_data,
                       headers=mock.ANY,
-                      cert=(None, None), verify=False),
-            mock.call(self.base_url + path,
-                      data=good_data,
-                      headers=mock.ANY,
                       cert=(None, None), verify=False)])
 
     @mock.patch('itertools.count')
-    @mock.patch('requests.patch')
+    @mock.patch('requests.sessions.Session.patch')
     def test_annotate_diff_resource_vers_no_annotation(self, m_patch, m_count):
         m_count.return_value = list(range(1, 5))
         path = '/test'
@@ -249,7 +250,7 @@ class TestK8sClient(test_base.TestCase):
                       cert=(None, None), verify=False)])
 
     @mock.patch('itertools.count')
-    @mock.patch('requests.patch')
+    @mock.patch('requests.sessions.Session.patch')
     def test_annotate_diff_resource_vers_conflict(self, m_patch, m_count):
         m_count.return_value = list(range(1, 5))
         path = '/test'
@@ -262,26 +263,38 @@ class TestK8sClient(test_base.TestCase):
         actual_obj = {'metadata': {
             'annotations': {'a1': 'v2'},
             'resourceVersion': new_resource_version}}
+        good_obj = {'metadata': {
+            'annotations': annotations,
+            'resourceVersion': new_resource_version}}
         conflicting_data = jsonutils.dumps(conflicting_obj, sort_keys=True)
+        good_data = jsonutils.dumps(good_obj, sort_keys=True)
 
         m_resp_conflict = mock.MagicMock()
         m_resp_conflict.ok = False
         m_resp_conflict.status_code = requests.codes.conflict
         m_patch.return_value = m_resp_conflict
+        m_resp_good = mock.MagicMock()
+        m_resp_good.ok = True
+        m_resp_good.json.return_value = conflicting_obj
+        m_patch.side_effect = [m_resp_conflict, m_resp_good]
 
         with mock.patch.object(self.client, 'get') as m_get:
             m_get.return_value = actual_obj
-            self.assertRaises(exc.K8sClientException,
-                              self.client.annotate,
-                              path, annotations,
-                              resource_version=resource_version)
-        m_patch.assert_called_once_with(self.base_url + path,
-                                        data=conflicting_data,
-                                        headers=mock.ANY,
-                                        cert=(None, None), verify=False)
+            self.assertEqual(annotations, self.client.annotate(
+                path, annotations,
+                resource_version=resource_version))
+        m_patch.assert_has_calls([
+            mock.call(self.base_url + path,
+                      data=conflicting_data,
+                      headers=mock.ANY,
+                      cert=(None, None), verify=False),
+            mock.call(self.base_url + path,
+                      data=good_data,
+                      headers=mock.ANY,
+                      cert=(None, None), verify=False)])
 
     @mock.patch('itertools.count')
-    @mock.patch('requests.patch')
+    @mock.patch('requests.sessions.Session.patch')
     def test_annotate_resource_not_found(self, m_patch, m_count):
         m_count.return_value = list(range(1, 5))
         path = '/test'
@@ -307,7 +320,7 @@ class TestK8sClient(test_base.TestCase):
                                         headers=mock.ANY,
                                         cert=(None, None), verify=False)
 
-    @mock.patch('requests.get')
+    @mock.patch('requests.sessions.Session.get')
     def test_watch(self, m_get):
         path = '/test'
         data = [{'obj': 'obj%s' % i} for i in range(3)]
@@ -328,9 +341,36 @@ class TestK8sClient(test_base.TestCase):
         self.assertEqual(cycles, m_resp.close.call_count)
         m_get.assert_called_with(self.base_url + path, headers={}, stream=True,
                                  params={'watch': 'true'}, cert=(None, None),
-                                 verify=False)
+                                 verify=False, timeout=(30, 60))
 
-    @mock.patch('requests.get')
+    @mock.patch('requests.sessions.Session.get')
+    def test_watch_restart(self, m_get):
+        path = '/test'
+        data = [{'object': {'metadata': {'name': 'obj%s' % i,
+                                         'resourceVersion': i}}}
+                for i in range(3)]
+        lines = [jsonutils.dump_as_bytes(i) for i in data]
+
+        m_resp = mock.MagicMock()
+        m_resp.ok = True
+        m_resp.iter_lines.side_effect = [lines, requests.ReadTimeout, lines]
+        m_get.return_value = m_resp
+
+        self.assertEqual(data * 2,
+                         list(itertools.islice(self.client.watch(path),
+                                               len(data) * 2)))
+        self.assertEqual(3, m_get.call_count)
+        self.assertEqual(3, m_resp.close.call_count)
+        m_get.assert_any_call(
+            self.base_url + path, headers={}, stream=True,
+            params={"watch": "true"}, cert=(None, None), verify=False,
+            timeout=(30, 60))
+        m_get.assert_any_call(
+            self.base_url + path, headers={}, stream=True,
+            params={"watch": "true", "resourceVersion": 2}, cert=(None, None),
+            verify=False, timeout=(30, 60))
+
+    @mock.patch('requests.sessions.Session.get')
     def test_watch_exception(self, m_get):
         path = '/test'
 
@@ -341,7 +381,7 @@ class TestK8sClient(test_base.TestCase):
         self.assertRaises(exc.K8sClientException, next,
                           self.client.watch(path))
 
-    @mock.patch('requests.post')
+    @mock.patch('requests.sessions.Session.post')
     def test_post(self, m_post):
         path = '/test'
         body = {'test': 'body'}
@@ -357,7 +397,7 @@ class TestK8sClient(test_base.TestCase):
                                        headers=mock.ANY, cert=(None, None),
                                        verify=False)
 
-    @mock.patch('requests.post')
+    @mock.patch('requests.sessions.Session.post')
     def test_post_exception(self, m_post):
         path = '/test'
         body = {'test': 'body'}
@@ -369,7 +409,7 @@ class TestK8sClient(test_base.TestCase):
         self.assertRaises(exc.K8sClientException,
                           self.client.post, path, body)
 
-    @mock.patch('requests.delete')
+    @mock.patch('requests.sessions.Session.delete')
     def test_delete(self, m_delete):
         path = '/test'
         ret = {'test': 'value'}
@@ -384,7 +424,7 @@ class TestK8sClient(test_base.TestCase):
                                          headers=mock.ANY, cert=(None, None),
                                          verify=False)
 
-    @mock.patch('requests.delete')
+    @mock.patch('requests.sessions.Session.delete')
     def test_delete_exception(self, m_delete):
         path = '/test'
 
@@ -394,3 +434,23 @@ class TestK8sClient(test_base.TestCase):
 
         self.assertRaises(exc.K8sClientException,
                           self.client.delete, path)
+
+    def test__raise_from_response(self):
+        m_resp = mock.MagicMock()
+        m_resp.ok = True
+        m_resp.status_code = 202
+        self.client._raise_from_response(m_resp)
+
+    def test__raise_from_response_404(self):
+        m_resp = mock.MagicMock()
+        m_resp.ok = False
+        m_resp.status_code = 404
+        self.assertRaises(exc.K8sResourceNotFound,
+                          self.client._raise_from_response, m_resp)
+
+    def test__raise_from_response_500(self):
+        m_resp = mock.MagicMock()
+        m_resp.ok = False
+        m_resp.status_code = 500
+        self.assertRaises(exc.K8sClientException,
+                          self.client._raise_from_response, m_resp)
